@@ -137,11 +137,16 @@ namespace xmlpp
 
 Document::Init::Init()
 {
-   xmlInitParser(); //Not always necessary, but necessary for thread safety.
-   xmlRegisterNodeDefault(on_libxml_construct);
-   xmlDeregisterNodeDefault(on_libxml_destruct);
-   xmlThrDefRegisterNodeDefault(on_libxml_construct);
-   xmlThrDefDeregisterNodeDefault(on_libxml_destruct);
+  xmlInitParser(); //Not always necessary, but necessary for thread safety.
+  xmlRegisterNodeDefault(on_libxml_construct);
+  xmlDeregisterNodeDefault(on_libxml_destruct);
+  xmlThrDefRegisterNodeDefault(on_libxml_construct);
+  xmlThrDefDeregisterNodeDefault(on_libxml_destruct);
+}
+
+Document::Init::~Init()
+{
+  xmlCleanupParser(); //As per xmlInitParser(), or memory leak will happen.
 }
 
 Document::Init Document::init_;
@@ -190,8 +195,8 @@ void Document::set_internal_subset(const Glib::ustring& name,
 {
   xmlDtd* dtd = xmlCreateIntSubset(impl_,
 				   (const xmlChar*)name.c_str(),
-				   (const xmlChar*)external_id.c_str(),
-				   (const xmlChar*)system_id.c_str());
+				   external_id.empty() ? (const xmlChar*)NULL : (const xmlChar*)external_id.c_str(),
+				   system_id.empty() ? (const xmlChar*)NULL : (const xmlChar*)system_id.c_str());
            
   if (dtd && !dtd->_private)
     dtd->_private = new Dtd(dtd);
@@ -224,12 +229,39 @@ Element* Document::create_root_node(const Glib::ustring& name,
   return element;
 }
 
+Element* Document::create_root_node_by_import(const Node* node,
+					      bool recursive)
+{
+  //Create the node, by copying:
+  xmlNode* imported_node = xmlDocCopyNode(const_cast<xmlNode*>(node->cobj()), impl_, recursive);
+  if (!imported_node)
+  {
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
+    throw exception("Unable to import node");
+    #else
+    return 0;
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
+  }
+
+  xmlDocSetRootElement(impl_, imported_node);
+
+  return get_root_node();
+}
+
 CommentNode* Document::add_comment(const Glib::ustring& content)
 {
   xmlNode* node = xmlNewComment((const xmlChar*)content.c_str());
   if(!node)
+  {
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
     throw internal_error("Cannot create comment node");
-  xmlAddChild( (xmlNode*)impl_, node);
+    #else
+    return 0;
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
+  }
+
+  // Use the result, because node can be freed when merging text nodes:
+  node = xmlAddChild( (xmlNode*)impl_, node);
   return static_cast<CommentNode*>(node->_private);
 }
 
@@ -275,7 +307,13 @@ void Document::do_write_to_file(
   result = xmlSaveFormatFileEnc(filename.c_str(), impl_, encoding.empty()?NULL:encoding.c_str(), format?1:0);
 
   if(result == -1)
+  {
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
     throw exception("do_write_to_file() failed.");
+    #else
+    return;
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
+  }
 }
 
 Glib::ustring Document::do_write_to_string(
@@ -290,10 +328,21 @@ Glib::ustring Document::do_write_to_string(
   xmlDocDumpFormatMemoryEnc(impl_, &buffer, &length, encoding.empty()?NULL:encoding.c_str(), format?1:0);
 
   if(!buffer)
+  {
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
     throw exception("do_write_to_string() failed.");
+    #else
+    return Glib::ustring();
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
+  }
 
   // Create a Glib::ustring copy of the buffer
-  Glib::ustring result((char*)buffer, length);
+
+  // Here we force the use of Glib::ustring::ustring( InputIterator begin, InputIterator end )
+  // instead of Glib::ustring::ustring( const char*, size_type ) because it
+  // expects the length of the string in characters, not in bytes.
+  Glib::ustring result( reinterpret_cast<const char *>(buffer), reinterpret_cast<const char *>(buffer + length) );
+
   // Deletes the original buffer
   xmlFree(buffer);
   // Return a copy of the string
@@ -312,7 +361,8 @@ void Document::set_entity_declaration(const Glib::ustring& name, XmlEntityType t
                               const Glib::ustring& content)
 {
   xmlAddDocEntity( impl_, (const xmlChar*) name.c_str(), type, 
-    (const xmlChar*) publicId.c_str(), (const xmlChar*) systemId.c_str(),
+    publicId.empty() ? (const xmlChar*)0 : (const xmlChar*)publicId.c_str(),
+    systemId.empty() ? (const xmlChar*)0 : (const xmlChar*)systemId.c_str(),
     (const xmlChar*) content.c_str() );
 }
 
@@ -321,4 +371,14 @@ _xmlEntity* Document::get_entity(const Glib::ustring& name)
   return xmlGetDocEntity(impl_, (const xmlChar*) name.c_str());
 }
 
+_xmlDoc* Document::cobj()
+{
+  return impl_;
+}
+
+const _xmlDoc* Document::cobj() const
+{
+  return impl_;
+}
+
 } //namespace xmlpp
